#    $Id: Alarm.pm,v 1.2 2005/07/07 17:24:31 kevinspicer Exp $
#    MailScanner-MRTG 
#    Copyright (C) 2002-3 Dale Lovelace <dlovelace@hotels.com>
#    With various bits shamelessly stolen from others
#    Restructuring and Substantial Modifications 
#    Copyright (C) 2003-5 Kevin Spicer <kevin@kevinspicer.co.uk>
#    with contributions from many other (see CHANGELOG for details)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Please submit all bug reports, patches etc via the sourceforge project 
#   page at http://sourceforge.net/projects/mailscannermrtg
#
#

package MSMRTG::Alarm;
require Exporter;

@ISA=qw(Exporter);
@EXPORT=qw(UpdateAlarmStatus WriteAlarmIncludes);

%alarminfo = (
  mail => { t1 => "mail-recipients", t2 => "mail-messages" },
  spam => { t1 => "spam-total", t2=> "spam-mta" },
  virus => { t1 => "virus-infected", t2=> "virus-virus" },
  mta => { t1 => "mta" },
  mailscanner => { t1 => "mailscanner" },
  mailbytes => { t1 => "mailbytes" },
  memory => { t1 => "memory-incache", t2=> "memory-excache" },
  inqueue => { t1 => "inqueue" },
  outqueue => { t1 => "outqueue" },
  quarantine => { t1 => "quarantine-files", t2 => "quarantine-messages" },
  loadavg => { t1 => "loadavg" },
  cpu => { t1 => "cpu-total", t2 => "cpu-user" },
  iptraffic => { t1 => "iptraffic-in", t2 => "iptraffic-out" },
  spoolusage => { t1 => "spoolusage" },
  tmpfsusage => { t1 => "tmpfsusage" },
  rootusage => { t1 => "rootusage" },
  spamratio => { t1 => "spamratio-total", t2 => "spamratio-mta" },
  virusratio => { t1 => "virusratio-infected", t2 => "virusratio-virus" },
  batchsize => { t1=> "batchsize" },
  speed => { t1 => "speed-total", t2=> "speed-spamchecks" }
);

sub GetAlarmStatus {
  # Sorts out the current alarm status from the state info
  # and puts into %alarminfo
  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Beginning sub GetAlarmStatus") ;
  }

  foreach ( keys %alarminfo) {
    $MSMRTG::State::logdata{$_}[2]=~/(NO|C[LH]|W[LH])(\d+)(?:(NO|C[LH]|W[LH])(\d+))?/;
	if (defined $1 && defined $2) { 
	  $alarminfo{$_}{s1}=$1;
	  $alarminfo{$_}{c1}=$2;
	} else {
	  # Otherwise assume normal 
	  $alarminfo{$_}{s1}="NO";
	  $alarminfo{$_}{c1}=0;
	}
	if (defined $alarminfo{$_}{t2}){
  	  if (defined $3 && defined $4 ) { 
	    $alarminfo{$_}{s2}=$3;
	    $alarminfo{$_}{c2}=$4;
	  } else {
	    # Otherwise assume normal 
	    $alarminfo{$_}{s2}="NO";
	    $alarminfo{$_}{c2}=0;
  	  }
	}
  }

  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Leaving sub GetAlarmStatus") ;
  }
}

sub SaveAlarmStatus {
  # Puts the current alarm status from %alarminfo
  # into state logdata
  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Beginning sub SaveAlarmStatus");
  }

  foreach ( keys %alarminfo) {
    $MSMRTG::State::logdata{$_}[2]=$alarminfo{$_}{s1}.$alarminfo{$_}{c1};
	if (defined $alarminfo{$_}{t2}){
      $MSMRTG::State::logdata{$_}[2].=$alarminfo{$_}{s2}.$alarminfo{$_}{c2};
	}
  }

  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Leaving sub SaveAlarmStatus") ;
  }
}

sub CheckAlarmState {
  # Returns the (new) alarm condition
  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Beginning sub CheckAlarmState");
  }
  my $category = shift;
  my $valuenum = shift;

  my $currentstate=$alarminfo{$category}{"s$valuenum"};
  my $threscat=$alarminfo{$category}{"t$valuenum"};
  my $currentval= $MSMRTG::State::logdata{$category}[$valuenum-1];

  # First establish the possible non-normal states.
  my %states= (
    CL => "critical_low",
	WL => "warning_low",
	WH => "warning_high",
	CH => "critical_high" 
  );

  foreach (qw(CL WL WH CH)) {
    my $thres="enter_" . $states{$_};
    unless ( defined $MSMRTG::Config::Thresholds{$threscat}{$thres} ) {
	  delete $states{$_};
	}
  }
	

  my $returnstate="NO";

  if (defined ($states{WL})){
    if ($currentstate eq "WL") { 
	  if ($currentval < 
	        $MSMRTG::Config::Thresholds{$threscat}{"leave_warning_low"} )
	  { $returnstate = "WL" }
	} else {
	  if ($currentval <= 
	        $MSMRTG::Config::Thresholds{$threscat}{"enter_warning_low"} )
	  { $returnstate = "WL" }
    }
  }
  if (defined ($states{CL})){
    if ($currentstate eq "CL") { 
	  if ($currentval < 
	        $MSMRTG::Config::Thresholds{$threscat}{"leave_critical_low"} )
	  { $returnstate = "CL" }
	} else {
	  if ($currentval <= 
	        $MSMRTG::Config::Thresholds{$threscat}{"enter_critical_low"} )
	  { $returnstate = "CL" }
    }
  }

  if (defined ($states{WH})){
    if ($currentstate eq "WH") { 
	  if ($currentval > 
	        $MSMRTG::Config::Thresholds{$threscat}{"leave_warning_high"} )
	  { $returnstate = "WH" }
	} else {
	  if ($currentval >= 
	        $MSMRTG::Config::Thresholds{$threscat}{"enter_warning_high"} )
	  { $returnstate = "WH" }
    }
  }

  if (defined ($states{CH})){
    if ($currentstate eq "CH") { 
	  if ($currentval > 
	        $MSMRTG::Config::Thresholds{$threscat}{"leave_critical_high"} )
	  { $returnstate = "CH" }
	} else {
	  if ($currentval >=
	        $MSMRTG::Config::Thresholds{$threscat}{"enter_critical_high"} )
	  { $returnstate = "CH" }
    }
  }
  

  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Leaving sub CheckAlarmState") ;
  }
  return ($returnstate);
}

sub UpdateAlarmStatus {
  # Checks all the existing thresholds against the current values
  # for each metric
  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Beginning sub UpdateAlarmStatus") ;
  }
  GetAlarmStatus();

  my $currentstate;
  foreach (keys(%alarminfo)){
    $currentstate=CheckAlarmState($_,1);
    if ( $currentstate eq $alarminfo{$_}{s1} )
      {$alarminfo{$_}{c1}++}
    else
      { $alarminfo{$_}{s1}=$currentstate; $alarminfo{$_}{c1}=1 }

    if (defined ($alarminfo{$_}{t2})) {
  	  $currentstate=CheckAlarmState($_,2);
      if ( $currentstate eq $alarminfo{$_}{s2} )
        {$alarminfo{$_}{c2}++}
      else
        { $alarminfo{$_}{s2}=$currentstate; $alarminfo{$_}{c2}=1 }
	}
  }

  SaveAlarmStatus();
  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Leaving sub UpdateAlarmStatus") ;
  }
}

sub WriteAlarmIncludes {
  # Puts the current alarm status from %alarminfo
  # into include files in the web server directories
  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Beginning sub WriteAlarmIncludes");
  }

  unless ( $MSMRTG::Config::Config{'wwwroot'} ) {
    MSMRTG::Log::Log_Syslog("err",0,"WWW Root not specified in $MSMRTG::Config::configfile, unable to write alarm includes");
	return;
  }

  unless ( -d $MSMRTG::Config::Config{'wwwroot'} ) {
    MSMRTG::Log::Log_Syslog("err",0,"WWW Root specified in $MSMRTG::Config::configfile is not a directory, unable to write alarm includes");
	return;
  }

  my ($category, $includedir, $includefile);
  foreach (keys(%MSMRTG::State::logdata)){
    $category=$_;
	if ($category eq "mta") { $category="sendmail" }
	elsif ($category eq "iptraffic") { $category="iptrafficnew" }
	$includedir=$MSMRTG::Config::Config{'wwwroot'} . "/mailscanner-mrtg/" . $category;
    unless ( -d $includedir ) {
      MSMRTG::Log::Log_Syslog("err",0,"$includedir is not a directory, unable to write alarm include");
	  next;
    }
	$includefile=$includedir . "/alarm.html";
	unless ( open INCLUDE, ">$includefile" ) {
      MSMRTG::Log::Log_Syslog("err",0,"Could not open $includefile, unable to write alarm include");
	  next;
    }
	if ($MSMRTG::State::logdata{$_}[2] =~ /C/) {
	  print INCLUDE "<td class=\"graphcrit\">\n";
	} elsif ($MSMRTG::State::logdata{$_}[2] =~ /W/) {
	  print INCLUDE "<td class=\"graphwarn\">\n";
	} else {
	  print INCLUDE "<td class=\"graph\">\n";
	}
	close INCLUDE;
  }

  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Leaving sub WriteAlarmIncludes") ;
  }
}
		
1;
