#    $Id: Config.pm,v 1.10 2005/07/03 17:00:56 kevinspicer Exp $
#    MailScanner-MRTG 
#    Copyright (C) 2002-3 Dale Lovelace <dlovelace@hotels.com>
#    With various bits shamelessly stolen from others
#    Restructuring and Substantial Modifications 
#    Copyright (C) 2003-5 Kevin Spicer <kevin@kevinspicer.co.uk>
#    with contributions from many other (see CHANGELOG for details)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Please submit all bug reports, patches etc via the sourceforge project 
#   page at http://sourceforge.net/projects/mailscannermrtg
#
#
package MSMRTG::Config;
require Exporter;

@ISA=qw(Exporter);
@EXPORT=qw($configfile $thresholdfile $version %timedate %Config ItoE ReadConfFile GetTimeDate %Thresholds ReadThresholdFile);

use FileHandle;
$configfile="";
$thresholdfile="";
$version="0";
@timedate=();
%Config=();
%Thresholds=();

#
# Read the configuration file. Borrowed from MailScanner :-) 
# These 3 subs are copyright Julian Field. Released under the GPL.
#
sub ItoE {
  my($val) = @_;
  lc($ItoE{$val}) or lc($val);
}
sub EtoI {
  my($val) = @_;
  lc($EtoI{$val}) or lc($val);
}
sub ReadConfFile {
  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Beginning sub ReadConfFile") ;
    MSMRTG::Debug::Debug("Configfile", $configfile);
  }

  # Slurp the whole file into a big hash.
  # Don't Complain if we see the same keyword more than once.
  my($fileh, $linecounter, $key, $value);

  $fileh = new FileHandle;
  $fileh->open($configfile) or
    MSMRTG::Log::Log_Syslog("err",1,"Could not read configuration file " . $configfile . " " .  $!);
  $linecounter = 0;
  while(<$fileh>) {
    $linecounter++;
    chomp;
    s/#.*$//;
    s/^\s+//;
    s/\s+$//;
    next if /^$/;

    $key = undef;
    $value = undef;
    /^(.*?)\s*=\s*(.*)$/;
    ($key,$value) = ($1,$2);

    $key = lc($key);
    $key =~ s/[^a-z0-9]//g; # Leave numbers and letters only

    # Translate the value to the internal (shorter) version of it
    $key = EtoI($key);

    $Config{$key} = $value;
    if ($MSMRTG::Debug::debug) { MSMRTG::Debug::Debug($key, $value) }
  }

	# If we found a regular expression for MTA name then we need to clean it up
	if ($Config{'mtaalternativename'}) {
		# Get rid of any anchors
		$Config{'mtaalternativename'} =~ s/^\^(.)/$1/g;
		$Config{'mtaalternativename'} =~ s/([^\\])[\$^]/$1/g;
		# Make any brackets non capturing
		$Config{'mtaalternativename'} =~ s/([^\\])\((?:\?:)?/$1(?:/g;
		# If we have still got a string left we wrap it up
		if (length($Config{'mtaalternativename'})) {
			$Config{'mtaalternativename'}="|(?:" . $Config{'mtaalternativename'} . ")";
		}
	} else { $Config{'mtaalternativename'} = "" }
  $fileh->close();
  if ($MSMRTG::Debug::debug) { MSMRTG::Debug::Debug("Leaving sub ReadConfFile") }
}

sub GetTimeDate{
	# This populates all the time related variables so we keep a consistent view 
	# about what the current time and date is.
  if ($MSMRTG::Debug::debug) { MSMRTG::Debug::Debug("Entering sub GetTimeDate") }

	$timedate{'utime'}=time();
  @timedate{qw(sec min hour mday mon year wday yday isdstr)} = localtime($timedate{'utime'});

	# Get text month
  $timedate{'month'} = (qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec))[$timedate{'mon'}]; 
	$timedate{'mon'}++;  # Make month no. 1-12 not 0-11
	$timedate{'year'}+=1900; # Proper 4 digit year

  if ($MSMRTG::Debug::debug){ 
		while (($key, $value) = each %timedate) {
						MSMRTG::Debug::Debug($key, $value);
		}

    MSMRTG::Debug::Debug("Leaving sub GetTimeDate"); 
  }
}

sub ReadThresholdFile {
  # Read the file containing the various alert thresholds
  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Beginning sub ReadThresholdFile") ;
    MSMRTG::Debug::Debug("Thresholdfile", $thresholdfile);
  }

  # Establish Defaults
  my ($metric, @metrics);
  @metrics=(qw(mail-messages mail-recipients spam-total spam-mta virus-infected virus-virus mta mailscanner mailbytes memory-excache memory-incache inqueue outqueue quarantine-files quarantine-messages loadavg cpu-total cpu-user iptraffic-in iptraffic-out spoolusage tmpfsusage rootusage spamratio-total spamratio-mta virusratio-infected virusratio-virus batchsize speed-total speed-spamchecks));
  foreach $metric (@metrics) {
  	$Thresholds{$metric} = {
				"scalefactor" => 1,
				"ispercentage" => 0,
	};
  }
  $Thresholds{loadavg}{scalefactor}=100;
  $Thresholds{"spamratio-total"}{ispercentage}=1;
  $Thresholds{"spamratio-mta"}{ispercentage}=1;
  $Thresholds{"virusratio-infected"}{ispercentage}=1;
  $Thresholds{"virusratio-virus"}{ispercentage}=1;
  $Thresholds{"cpu-total"}{ispercentage}=1;
  $Thresholds{"cpu-user"}{ispercentage}=1;

  # Read the file
  my($fileh, $tempmetric, %tempthres);

  $fileh = new FileHandle;
  unless ($fileh->open($thresholdfile)) {
    MSMRTG::Log::Log_Syslog("err",0,"Could not read thresholds file " . $thresholdfile . " " .  $!);
	return;
  }
  while(<$fileh>) {
    # Discard comment and blank lines
	chomp;
	if (/^\s*#/ || /^\s*$/) {next}

	if (defined $tempmetric) {
	
	  if ( /}/ ) {
	    commitThresholds($tempmetric,\%tempthres);
		undef $tempmetric;
	  } else {

        if ( /enter_critical_low\s*=\s*([\d.]+)/ )
    	    {$tempthres{enter_critical_low}=$1 }
        if ( /leave_critical_low\s*=\s*([\d.]+)/ )
	      {$tempthres{leave_critical_low}=$1 }
        if ( /enter_warning_low\s*=\s*([\d.]+)/ )
	      {$tempthres{enter_warning_low}=$1 }
        if ( /leave_warning_low\s*=\s*([\d.]+)/ )
	      {$tempthres{leave_warning_low}=$1 }
        if ( /leave_warning_high\s*=\s*([\d.]+)/ )
	      {$tempthres{leave_warning_high}=$1 }
        if ( /enter_warning_high\s*=\s*([\d.]+)/ )
	      {$tempthres{enter_warning_high}=$1 }
        if ( /leave_critical_high\s*=\s*([\d.]+)/ )
	      {$tempthres{leave_critical_high}=$1 }
        if ( /enter_critical_high\s*=\s*([\d.]+)/ )
    	    {$tempthres{enter_critical_high}=$1 }

	  }
    } elsif (/^([\w-]+)\s+{/) { 

	  if (defined ($Thresholds{$1})) {
	    $tempmetric=$1;
		%tempthres=();
	  } else {
	    MSMRTG::Log::Log_Syslog("err",0,"Unrecognised metric $1 in thresholds file");
	  }

	}
  }

  close ($fileh);

  if ($MSMRTG::Debug::debug) { 
    foreach $metric (keys(%Thresholds)) {
	  foreach (keys(%{$Thresholds{$metric}})) {
	    if ($_ eq "scalefactor" || $_ eq "ispercentage") { next }
	    MSMRTG::Debug::Debug("Threshold $_ for $metric is " . $Thresholds{$metric}{$_});
	  }
	}
    MSMRTG::Debug::Debug("Leaving sub ReadThresholdFile") ;
  }
}

sub commitThresholds {
  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Entering sub commitThresholds") ;
  }
  my $tempmetric=shift;
  my $tempthres=shift;
  my $thres;

  # If a percentage value, check is <=100
  if ( $Thresholds{$tempmetric}{ispercentage} == 1 ){
    foreach $thres (keys(%$tempthres)) {
	  if ($$tempthres{$thres} > 100) {
	    MSMRTG::Log::Log_Syslog("err",0,"Percentage Threshold may not be more than 100 for threshold: $thres metric: $tempmetric");
		return;
	  }
	}
  }

  foreach $thres (qw(critical_low warning_low warning_high critical_high)){
  
    # If 'leave' defined check 'enter' is too
	if (defined ($$tempthres{"leave_$thres"}) &&
	      !defined ($$tempthres{"enter_$thres"}) ) {
	  MSMRTG::Log::Log_Syslog("err",0,"May not define leave_$thres without enter_$thres for metric: $tempmetric");
	  return;
	}
	# If 'enter' defined but 'leave' not then set 'leave'
	if (defined ($$tempthres{"enter_$thres"}) &&
	      !defined ($$tempthres{"leave_$thres"})) {
	  $$tempthres{"leave_$thres"} = $$tempthres{"enter_$thres"};
	}
  }

  # Now check the thresholds are in ascending order
  my $lastval;
  my $lastthres;
  my $count=0;
  foreach $thres (qw(enter_critical_low leave_critical_low enter_warning_low leave_warning_low leave_warning_high enter_warning_high leave_critical_high enter_critical_high)) {
    $count++;
    unless (defined ($$tempthres{$thres})) { next; }
	unless (defined $lastval) { 
	  $lastval = $$tempthres{$thres};
	  $lastthres = $thres;
	  next;
	}
	# for pairs of values = is okay
	if ( $count % 2 == 0 && $lastval == $$tempthres{$thres} ) {
	  $lastval=$$tempthres{$thres};
	  $lastthres = $thres;
	  next;
	}
	# Otherwise check its higher...
	if ( $$tempthres{$thres} <= $lastval ){
	  MSMRTG::Log::Log_Syslog("err",0,"$thres is not higher than $lastthres for metric: $tempmetric");
	  return;
	}
	$lastval=$$tempthres{$thres};
	$lastthres = $thres;
  }

  # Checks complete, commit the values
  foreach $thres (keys(%$tempthres)) {
    $Thresholds{$tempmetric}{$thres} = $$tempthres{$thres};
  }

  if ($MSMRTG::Debug::debug) { 
    MSMRTG::Debug::Debug("Leaving sub commitThresholds") ;
  }
}
1;
